#include <iostream>
#include <fstream>
#include <vector>
#include <stdlib.h>  
#include <cstring>
#include <string>
#include <sstream>

#include <lemon/list_graph.h>
#include <lemon/dijkstra.h>
#include <lemon/lgf_reader.h>
#include <lemon/lgf_writer.h>
#include <lemon/core.h>
#include <lemon/maps.h>
#include <lemon/path.h>
#include <lemon/concepts/path.h>
#include <lemon/connectivity.h>
#include <lemon/euler.h>
using namespace lemon;


/////////////////////////////////////////////////////  /* INPUT */    ////////////////////////////////////////////////////////////////////////////////////////////



//TSP
     // Finds a tour that visits only all cities if the boolean is true, otherwise the tour will also visit all connections



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// Initialize
int biggestgroup;
int NumClusters = 0;
int Connections;
int ExtraConnections = 0;
int Msum = 0;
int Mmax = 0;
int bigM = 0;
int Mmax2 = 0;
int bigM2 = 0;


/* Define Starting Graph */
  ListDigraph GS;
  ListDigraph::NodeMap<std::string> Sname(GS);
  ListDigraph::NodeMap<int> Sstation(GS);
  ListDigraph::ArcMap<int> Slength(GS);
  ListDigraph::ArcMap<int> Sdirection(GS);
  ListDigraph::ArcMap<int> Sline(GS);

/* Define GDRPP Graph */
  ListDigraph G;
  ListDigraph::NodeMap<std::string> Gname(G);
  ListDigraph::NodeMap<int> Gline(G);
  ListDigraph::NodeMap<int> Gdirection(G);
  ListDigraph::NodeMap<int> Gstation(G);
  ListDigraph::NodeMap<int> Gstart(G);
  ListDigraph::NodeMap<int> Gfreq(G);
  ListDigraph::ArcMap<int> Glength(G);
  ListDigraph::ArcMap<int> Ggroup(G);
  ListDigraph::ArcMap<int> Garcline(G);
  ListDigraph::ArcMap<int> Garcdirection(G);

/* Define GDRPP Extended Graph */
  ListDigraph GT;
  ListDigraph::NodeMap<std::string> name(GT);
  ListDigraph::NodeMap<int> line(GT);
  ListDigraph::NodeMap<int> direction(GT);
  ListDigraph::NodeMap<int> station(GT);
  ListDigraph::NodeMap<int> time2(GT);
  ListDigraph::NodeMap<int> originalid(GT);
  ListDigraph::NodeMap<int> freq(GT);
  ListDigraph::ArcMap<int> length(GT);
  ListDigraph::ArcMap<int> group(GT);
  ListDigraph::ArcMap<int> arcline(GT);
  ListDigraph::ArcMap<int> arcdirection(GT);
  ListDigraph::ArcMap<int> arctime(GT);
  ListDigraph::ArcMap<bool> nontransfer(GT);
  ListDigraph::ArcMap<int> idG(GT);

/* Define GATSP Graph */
  ListDigraph GATSP;
  ListDigraph::NodeMap<int> nodegroup(GATSP);
  ListDigraph::NodeMap<int> nodeline(GATSP);
  ListDigraph::NodeMap<int> nodelength(GATSP);
  ListDigraph::NodeMap<int> nodedirection(GATSP);
  ListDigraph::NodeMap<int> nodetime(GATSP);
  ListDigraph::NodeMap<ListDigraph::Node> nodesource(GATSP);
  ListDigraph::NodeMap<ListDigraph::Node> nodetarget(GATSP);
  ListDigraph::NodeMap<ListDigraph::Node> prec(GATSP);
  ListDigraph::NodeMap<int> arcidG(GATSP);
  ListDigraph::ArcMap<int> arclength(GATSP);

/* Define ATSP Graph */
  ListDigraph ATSP;
  ListDigraph::NodeMap<int> cluster(ATSP);
  ListDigraph::NodeMap<int> newline(ATSP);
  ListDigraph::NodeMap<int> newdirection(ATSP);
  ListDigraph::NodeMap<int> order(ATSP);
  ListDigraph::NodeMap<int> maxorder(ATSP);
  ListDigraph::NodeMap<int> newtime(ATSP);
  ListDigraph::NodeMap<int> newarcidG(ATSP);
  ListDigraph::ArcMap<int> newlength(ATSP);

/* Define TSP Graph */
  ListDigraph TSP;
  ListDigraph::NodeMap<int> defcluster(TSP);
  ListDigraph::NodeMap<int> defline(TSP);
  ListDigraph::NodeMap<int> defdirection(TSP);
  ListDigraph::NodeMap<bool> original(TSP);
  ListDigraph::NodeMap<ListDigraph::Node> originalnode(TSP);
  ListDigraph::ArcMap<int> deflength(TSP);

/* Define Solution Graph */
  ListDigraph SG;
  ListDigraph::NodeMap<std::string> solname(SG);
  ListDigraph::NodeMap<int> nsolline(SG);
  ListDigraph::NodeMap<int> nsoldirection(SG);
  ListDigraph::NodeMap<int> soltime(SG);
  ListDigraph::NodeMap<int> solfreq(SG);
  ListDigraph::ArcMap<int> sollength(SG);
  ListDigraph::ArcMap<int> asolline(SG);
  ListDigraph::ArcMap<int> asoldirection(SG);
  ListDigraph::ArcMap<int> solgroup(SG);
  ListDigraph::ArcMap<int> solorder(SG);
  ListDigraph::ArcMap<bool> solnontransfer(SG);
  ListDigraph::ArcMap<std::string> solsource(SG);
  ListDigraph::ArcMap<std::string> soltarget(SG);
  ListDigraph::ArcMap<int> solsourcetime(SG);
  ListDigraph::ArcMap<int> soltargettime(SG);

int concatenate(int x, int y)
{
    int pow = 10;
    int small = x;
    int big = y;
    if (x > y){
        small = y;
        big = x;
    }
    while(big >= pow){
        pow *= 10;
    }
    return small * pow + big;
}

void GSTART(std::string inptst)
{
/* Read start graph from file */
    digraphReader(GS, inptst).
      nodeMap("name", Sname).
      nodeMap("station", Sstation).
      arcMap("length", Slength).
      arcMap("direction", Sdirection).
      arcMap("line", Sline).
      run();

    int stations = countNodes(GS);
    biggestgroup = concatenate(stations, stations);

/* Report results */
  std::cout << "The starting graph is constructed" << std::endl;
  std::cout << "Number of nodes: " << countNodes(GS) << std::endl;
  std::cout << "Number of arcs: " << countArcs(GS) << std::endl;
}
void GSTARTtoGDRPP(std::string inputGDRPP, bool TSPvar)
{
/* Read GDRPP nodes from file */
    digraphReader(G, inputGDRPP).
      nodeMap("name", Gname).
      nodeMap("station", Gstation).
      nodeMap("line", Gline).
      nodeMap("direction", Gdirection).
      nodeMap("start", Gstart).
      nodeMap("frequency", Gfreq).
      run();

/* Add GDRPP connection arcs */
int checkadded;
  for (ListDigraph::ArcIt a(GS); a!= INVALID; ++a){
    checkadded = 0;
    for (ListDigraph::NodeIt n(G); n!= INVALID; ++n){
        for (ListDigraph::NodeIt m(G); m!= INVALID; ++m){
            if((Gstation[n] == Sstation[GS.source(a)]) && (Gstation[m] == Sstation[GS.target(a)]) && (Sline[a]==Gline[n]) && (Sline[a]==Gline[m]) && (Sdirection[a]==Gdirection[n]) && (Sdirection[a] == Gdirection[m])){
                ListDigraph::Arc x = G.addArc(n,m);
                checkadded++;
                Glength[x] = Slength[a];
                if (Gline[n] == Gline[m]){
                    Garcline[x] = Gline[n];
                }
                else {
                    std::cout<<"Line error"<<std::endl;
                }
                if (Gdirection[n] == Gdirection[m]){
                    Garcdirection[x] = Gdirection[n];
                }
                else{
                    std::cout<<"Direction error"<<std::endl;
                }
            }
        }
    }
    if (checkadded != 1){
        std::cout<<GS.id(a)<<" added wrongly"<<std::endl;
    }
  }

/* Add GDRPP group labels */
  for (ListDigraph::ArcIt a(G); a != INVALID; ++a){
      for(ListDigraph::NodeIt n(G); n!= INVALID; ++n){
          for (ListDigraph::NodeIt m(G); m!= INVALID; ++m){
              if((G.id(G.source(a))==G.id(n)) && (G.id(G.target(a))==G.id(m))){
                if (Garcline[a]%10 != 0){         //Extra connections have a line of 0 or 10
                    Ggroup[a] = concatenate(Gstation[n], Gstation[m]);
                  }
                  else{
                    Ggroup[a] = 0;
                  }
              }
          }
      }
  }

// Count groups and nodes in groups
if (TSPvar == false){
  int allgroups[biggestgroup];
  for (int i=0; i<biggestgroup; ++i){
    allgroups[i] = 0;
  }
  for (ListDigraph::ArcIt a(G); a!= INVALID; ++a){
    if (Ggroup[a]!=0){
        ++allgroups[Ggroup[a]];
    }
  }

  for (int i=0; i<biggestgroup; ++i){
    if (allgroups[i] != 0){
        ++NumClusters;
    }
  }
}
else{
    NumClusters = countNodes(GS);
}

//Report results
  std::cout << "GDRPP is constructed." << std::endl;
  std::cout << "Number of nodes: " << countNodes(G) << std::endl;
  std::cout << "Number of arcs: " << countArcs(G) << std::endl;
  //std::cout<< "NumberOfGroups: "<< NumClusters<< std::endl;

  /* Check GDRPP transformation */
  if( countArcs(G) == countArcs(GS)){
    std::cout<<"Transformation to GDRPP correct"<< std::endl;
  }
  else{
    std::cout<<"Incorrect transformation: Original # of arcs =" << countArcs(GS)<< "New # of arcs = " << countArcs(G) << std::endl;
  }

// write G to the standard output
  digraphWriter(G, "GDRPP.lgf").
    nodeMap("name", Gname).
    nodeMap("station", Gstation).
    nodeMap("line", Gline).
    nodeMap("direction", Gdirection).
    nodeMap("frequency", Gfreq).
    arcMap("length", Glength).
    arcMap("group", Ggroup).
    arcMap("line", Garcline).
    run();
}

void GDRPPtoGDRPPextended(int timeInterval, int Transfertime, int lbTransfer, std::vector<int[2]> exceptionStations, int unwantedTransfer, int weightUnwantedTransfer)
{
/* Duplicate nodes for times */
    int NumTimes = 0;
    for (ListDigraph::NodeIt n(G); n!= INVALID; ++n){
        if (Gfreq[n] > timeInterval){
            Gfreq[n] = timeInterval;
        }
        if (timeInterval == 1){
            NumTimes = 1;
        }
        else{
            NumTimes = timeInterval/Gfreq[n];
        }
        for (int i = 0; i< NumTimes; i++){
            ListDigraph::Node x = GT.addNode();
            if (timeInterval != 1){
                time2[x] = (Gstart[n] + i*Gfreq[n])%timeInterval;
            }
            else{
                time2[x] = Gstart[n];
            }
            freq[x] = Gfreq[n];
            line[x] = Gline[n];
            direction[x] = Gdirection[n];
            station[x] = Gstation[n];
            originalid[x] = G.id(n);
            name[x] = Gname[n];
        }
    }

//Calculate how many nodes there should be
  int GTnodes = 0;
  if (timeInterval != 1){
    for (ListDigraph::NodeIt n(G); n!= INVALID; ++n){
            GTnodes = GTnodes + (timeInterval/Gfreq[n]);
    }
  }
  else{
    GTnodes = countNodes(G);
  }

/* Add connection arcs */
 // for each arc in G add the arc between the right nodes in GT -> note:
 // 1: id source and target, 2: length in comparison to time difference
  bool checkadded;
  int temp1=0;
  int temp2=0;
  int diff;
  int diffnew;
  int lengthnew;
  for (ListDigraph::ArcIt a(G); a!= INVALID; ++a){
    if (timeInterval == 1){
        temp1 = 1;
    }
    else{
        temp1 = std::min(timeInterval/Gfreq[G.source(a)], timeInterval/Gfreq[G.target(a)]);
    }
    temp2 = 0;
    checkadded = false;
    Connections = Connections + temp1;        //check number of arcs that should be added
    if (Ggroup[a] == 0){
        ExtraConnections = ExtraConnections + temp1;             //count extra connections that should be added
    }
    for (ListDigraph::NodeIt n(GT); n!= INVALID; ++n){
        for (ListDigraph::NodeIt m(GT); m!= INVALID; ++m){
            if ((G.id(G.source(a)) == originalid[n]) &&(G.id(G.target(a)) == originalid[m])){       //only add arc if it's between right nodes
                diff = time2[m] - time2[n];
                if (diff >= 0){
                    diffnew = diff;
                }
                else{
                    diffnew = timeInterval + diff;
                }
                lengthnew = Glength[a]%timeInterval;
                if ((diffnew == lengthnew) || timeInterval == 1){                    //only add arc if length is corresponding
                    ListDigraph::Arc x = GT.addArc(n,m);
                    length[x] = Glength[a];
                    group[x] = Ggroup[a];
                    arcline[x] = Garcline[a];
                    arcdirection[x] = Garcdirection[a];
                    nontransfer[x] = true;
                    checkadded = true;
                    idG[x] = G.id(a);
                    temp2++;                }
            }
        }
    }
    if (checkadded == false){
        std::cout<<"Arc "<<G.id(a)<<" not added "<<G.id(G.source(a))<<"-"<<G.id(G.target(a))<<" Length "<<diffnew<<"-"<<lengthnew<<std::endl;
    }
    if (temp1 != temp2){
        std::cout<<G.id(a)<<" - "<<temp1<<"-"<<temp2<<std::endl;
    }
  }

  int GTarcs = countArcs(GT);

//Define arc times
  for (ListDigraph::ArcIt a(GT); a!= INVALID; ++a){
    arctime[a] = time2[GT.source(a)];
  }

/* Add transfer arcs */
// Determine transfer times
if (timeInterval != 1){
  int transfer;
  bool shouldAdd;
  for (ListDigraph::NodeIt n(GT); n != INVALID; ++n){
    for (ListDigraph::NodeIt m(GT); m!= INVALID; ++m){
        if ((station[n] == station[m]) && (GT.id(n)!=GT.id(m))){
            shouldAdd = true;
            transfer = time2[m] - time2[n];
            if (transfer < lbTransfer ){                                // Make transfer times bigger than the stated lower bound (not negative & zero time is not possible
                transfer = timeInterval + transfer;
            }
            if (transfer == unwantedTransfer){                         // Give small transfer times a heavier weight
                transfer = transfer + weightUnwantedTransfer;
            }
            for (int i=0; i<exceptionStations.size(); ++i){                    // Take exceptions of big stations into account
                if ((station[n] == exceptionStations.at(i)[0]) && (transfer < exceptionStations.at(i)[1])){
                    shouldAdd = false;
                }
            }
            if (shouldAdd == true){
                ListDigraph::Arc x = GT.addArc(n,m);
                length[x] = transfer;
                nontransfer[x] = false;
            }
        }
    }
  }
}   /* Add constant GDRPP transfer arcs */
else{
  for (ListDigraph::NodeIt n(GT); n != INVALID; ++n){
    for (ListDigraph::NodeIt m(GT); m!= INVALID; ++m){
        if ((station[n] == station[m]) && (GT.id(n)!=GT.id(m))){
            ListDigraph::Arc x = GT.addArc(n,m);
            length[x] = Transfertime;
            nontransfer[x] = false;
        }
    }
  }
}

//Report results
  std::cout << "GDRPP Extended is constructed." << std::endl;
  std::cout << "Number of nodes: " << countNodes(GT) << std::endl;
  std::cout << "Number of arcs: " << countArcs(GT) << std::endl;
  std::cout<< "NumberOfGroups: "<< NumClusters<< std::endl;

  /* Check GDRPP transformation */
  if((Connections == GTarcs) && (countNodes(GT) == GTnodes)){
    std::cout<<"Transformation to GDRPP Extended correct"<< std::endl;
  }
  else{
    std::cout<<"Incorrect transformation: Original # of arcs =" << GTarcs << " New # of arcs = " << Connections <<" New # nodes = "<<GTnodes<< std::endl;
  }
// write GT to the standard output
    digraphWriter(GT, "GT.lgf").
    nodeMap("name", name).
    nodeMap("station", station).
    nodeMap("line", line).
    nodeMap("direction", direction).
    nodeMap("frequency", freq).
    nodeMap("time", time2).
    arcMap("length", length).
    arcMap("group", group).
    run();

}

void GDRPPtoGATSP(bool TSPvar)
{
  int intracluster = 0;
if (TSPvar == false){
/* Add nodes GATSP */
  for (ListDigraph::ArcIt a(GT); a!= INVALID; ++a){
    if (group[a] != 0){
        ListDigraph::Node x = GATSP.addNode();
        nodegroup[x] = group[a];
        nodelength[x] = length[a];
        nodesource[x] = GT.source(a);
        nodetarget[x] = GT.target(a);
        nodeline[x] = arcline[a];
        nodedirection[x] = arcdirection[a];
        nodetime[x] = arctime[a];
        arcidG[x] = idG[a];
    }
  }

/* Add arcs GATSP */
  ListDigraph::NodeMap<int> dist(GT);
  Dijkstra<ListDigraph> dijkstra(GT, length);
  dijkstra.distMap(dist);
  int cnt = 0;
  for (ListDigraph::NodeIt i(GATSP); i!= INVALID; ++i){
    ListDigraph::Node n = nodetarget[i];
    dijkstra.run(n);
    for (ListDigraph::NodeIt j(GATSP); j!= INVALID; ++j){
        ListDigraph::Node m = nodesource[j];
        if (nodegroup[i] != nodegroup[j]){              //Only arcs between different clusters
            ListDigraph::Arc x = GATSP.addArc(i,j);
            arclength[x] = dist[m] + nodelength[i];     //Update GATSP with lengths of G & Add length of target node edge
            //Msum = Msum + arclength[x];               //Calculate total sum
            if (arclength[x] > Mmax){
                Mmax = arclength[x];                    //Find maximum arc length
            }
        }
    }
    cnt++;
  }
}
else{
/* Add nodes GATSP */
  digraphReader(GATSP, "GT.lgf").
    nodeMap("station", nodegroup).
    nodeMap("line", nodeline).
    nodeMap("direction", nodedirection).
    nodeMap("time", nodetime).
    skipArcs().
    run();

/* Add arcs GATSP */
  ListDigraph::NodeMap<int> dist(GT);
  Dijkstra<ListDigraph> dijkstra(GT, length);
  dijkstra.distMap(dist);
  for (ListDigraph::NodeIt i(GATSP); i!= INVALID; ++i){
    dijkstra.run(i);
    for (ListDigraph::NodeIt j(GATSP); j!= INVALID; ++j){
        if (station[i] != station[j]){              //Only arcs between different clusters
            ListDigraph::Arc x = GATSP.addArc(i,j);
            arclength[x] = dist[j];                 //Update GATSP with lengths of G
            if (arclength[x] > Mmax){
                Mmax = arclength[x];                    //Find maximum arc length
            }
        }
    }
  }
// Calculate number of arcs
    int clusterelements[NumClusters];
    for (int i=1; i<NumClusters+1; i++){
        clusterelements[i] = 0;
    }
    for (ListDigraph::NodeIt n(GATSP); n!= INVALID; ++n){
        clusterelements[nodegroup[n]] ++;
    }
    for (int i= 1; i<NumClusters+1; i++){
        intracluster = intracluster + clusterelements[i]*(clusterelements[i]-1);
    }
}

//Report results
  std::cout << "GATSP is constructed" << std::endl;
  std::cout << "Number of nodes: " << countNodes(GATSP) << std::endl;
  std::cout << "Number of arcs: " << countArcs(GATSP) << std::endl;
  std::cout << "Number of groups: " << NumClusters << std::endl;

//Check transformation
if (TSPvar == false){
  if( Connections - ExtraConnections == countNodes(GATSP)){
    std::cout<<"Transformation GDRPP to GATSP correct"<< std::endl;
  }
  else{
    std::cout<<"Incorrect transformation: Original # of arcs =" << Connections - ExtraConnections << " New # of nodes = " << countNodes(GATSP)<< std::endl;
  }
}
else{
    if (countNodes(GATSP)*(countNodes(GATSP)-1) - intracluster == countArcs(GATSP)){
        std::cout<<"Transformation GDRPP to GATSP for TSP correct"<<std::endl;
    }
    else{
        std::cout<<"Incorrect transformation: Intracluster arcs ="<< intracluster<<std::endl;
    }
}

    digraphWriter(GATSP, "GATSP.lgf").
    nodeMap("group", nodegroup).
    nodeMap("line", nodeline).
    nodeMap("direction", nodedirection).
    nodeMap("time", nodetime).
    nodeMap("id", arcidG).
    arcMap("length", arclength).
    run();

}

void GATSPtoATSP(bool open, bool TSPvar, int startingpoint, int endingpoint)
{
  
/* Add nodes to ATSP graph */
    digraphReader(ATSP, "GATSP.lgf").
      nodeMap("group", cluster).
      nodeMap("line", newline).
      nodeMap("direction", newdirection).
      nodeMap("time", newtime).
      nodeMap("id", newarcidG).
      skipArcs().
      run();

// Make list with groups and extract group info
int clusterinfo[NumClusters][2];
if (TSPvar == false){
  int allgroups[biggestgroup];
  for (int i=0; i<biggestgroup; ++i){
    allgroups[i] = 0;
  }
  for (ListDigraph::ArcIt a(GT); a!= INVALID; ++a){
    if (group[a]!=0){
        ++allgroups[group[a]];
    }
  }
  int j=0;
  for (int i=0; i< NumClusters; ++i){
    while (allgroups[j] == 0){
        j++;
    }
    clusterinfo[i][0] = j;              //name of cluster
    clusterinfo[i][1] = allgroups[j];   //# cluster elements
    j++;
  }
}
else{
    for (int i=0; i<NumClusters; i++){
        clusterinfo[i][0] = i+1;
        clusterinfo[i][1] = 0;
    }
    for (ListDigraph::NodeIt n(GT); n!= INVALID; ++n){
        clusterinfo[station[n]-1][1]++;
    }
}

// Define orders to nodes
  int cntorder = 1;
  for (int i = 0; i<NumClusters; ++i){
    cntorder = 1;
    for (ListDigraph::NodeIt n(ATSP); n!= INVALID; ++n){
        if (cluster[n] == clusterinfo[i][0]){
            order[n] = cntorder;
            maxorder[n] = clusterinfo[i][1];
            cntorder++;
        }
    }
  }

/* Add cycle arcs with length 0 and define predecessors */
  for (ListDigraph::NodeIt n(ATSP); n!= INVALID; ++n){
    for (ListDigraph::NodeIt m(ATSP); m!= INVALID; ++m){
        if ((cluster[n] == cluster[m]) &&((order[m]-order[n] == 1)||(order[n]-order[m]==maxorder[n]-1))){
            newlength[ATSP.addArc(n,m)] = 0;
            prec[m] = n;
        }
    }
  }

/* Add connection arcs */
   bigM = Mmax*NumClusters;
   ArcLookUp<ListDigraph> al1(GATSP);
  for (ListDigraph::NodeIt n(GATSP); n!= INVALID; ++n){
    for (ListDigraph::NodeIt m(GATSP); m!= INVALID; ++m){
        if (nodegroup[n] != nodegroup[m]){
            for (ListDigraph::NodeIt k(GATSP); k!= INVALID; ++k){
                if (k==prec[n]){
                    ListDigraph::Arc x = ATSP.addArc(k,m);
                    ListDigraph::Arc y = al1(n,m);
                    if (y!= INVALID){
                        newlength[x] = arclength[y] + bigM;     // Include large constant
                    }
                    else{
                        std::cout<<"ERROR invalid arc"<<std::endl;
                    }
                }
            }
        }
    }
  }

// Add starting node for open tour with defined starting point
  int extra = 0;
  int extra2 = 0;
  if ((open == true) && ((startingpoint != 0) || (endingpoint != 0))){
    ListDigraph::Node xx = ATSP.addNode();
    extra2 = 1;
    if (TSPvar == false){
        for (ListDigraph::NodeIt s(G); s!= INVALID; ++s){
        // Find all outgoing arcs of starting point
            if ((Gstation[s] == startingpoint) && (startingpoint != 0)){
                for (ListDigraph::OutArcIt a(G,s); a!= INVALID; ++a){
                    for (ListDigraph::NodeIt n(ATSP); n!= INVALID; ++n){
                        if (newarcidG[n] == G.id(a)){
                            newlength[ATSP.addArc(xx,n)] = 0;
                            extra++;
                        }
                    }
                }
            }
        // Find all ingoing arcs of ending point
            if ((Gstation[s] == endingpoint) && (endingpoint != 0)){
                for (ListDigraph::InArcIt a(G,s); a!= INVALID; ++a){
                    for (ListDigraph::NodeIt n(ATSP); n!= INVALID; ++n){
                        if (newarcidG[n] == G.id(a)){
                            newlength[ATSP.addArc(n,xx)] = 0;
                            extra++;
                        }
                    }
                }
            }
        }
    }
    else{
        for (ListDigraph::NodeIt n(ATSP); n!= INVALID; ++n){
            if ((cluster[n] == startingpoint) && (startingpoint != 0)){
                newlength[ATSP.addArc(xx,n)] = 0;
                extra++;
            }
            if ((cluster[n] == endingpoint) && (endingpoint != 0)){
                newlength[ATSP.addArc(n,xx)] = 0;
                extra++;
            }
        }
    }
  }


//Report results
  std::cout << "ATSP is constructed" << std::endl;
  std::cout << "Number of nodes: " << countNodes(ATSP) << std::endl;
  std::cout << "Number of arcs: " << countArcs(ATSP) << std::endl;

//Check transformation
  if( (countNodes(GATSP) + extra2 == countNodes(ATSP))&&(countArcs(GATSP)+countNodes(GATSP) + extra==countArcs(ATSP))){
    std::cout<<"Transformation GATSP to ATSP correct"<< std::endl;
  }
  else{
    std::cout<<"Incorrect transformation: "<<countArcs(GATSP) <<"+"<<countNodes(GATSP)  <<"+"<< extra<<" should equal "<<countArcs(ATSP)<<std::endl;
        //Original # of arcs =" << countArcs(GATSP)<<", original # of nodes="<<countNodes(GATSP) << " New # of arcs = " << countArcs(ATSP)<< std::endl;
  }

  digraphWriter(ATSP, "ATSP.lgf").
    nodeMap("cluster", cluster).
    nodeMap("order", order).
    nodeMap("maxorder", maxorder).
    nodeMap("line", newline).
    nodeMap("direction", newdirection).
    arcMap("length", newlength).
    run();

}

void ATSPtoTSP(bool open, int startingpoint, int endingpoint)
{
/* Add nodes to TSP graph */
    digraphReader(TSP, "ATSP.lgf").
      skipArcs().
      run();

// Find node info
  int SizeTSP = countNodes(TSP);
  for (int i = 0; i< SizeTSP; i++){
    for (ListDigraph::NodeIt n(TSP); n!= INVALID; ++n){
        if (TSP.id(n) == i){
            originalnode[n] = n;
            original[n] = true;
            ListDigraph::Node y = TSP.addNode();
            originalnode[y] = n;
            original[y] = false;
        }
    }
}

// Find maximum length in ATSP
  for (ListDigraph::ArcIt a(ATSP); a!= INVALID; ++a){
    Msum = Msum + newlength[a];
    if (newlength[a] > Mmax2){
        Mmax2 = newlength[a];
    }
  }
  bigM2 = Mmax2;


// Set edges u,v' to u,v
  ArcLookUp<ListDigraph> al2(ATSP);
  for (ListDigraph::NodeIt n(TSP); n!= INVALID; ++n){
    ListDigraph::Node x = originalnode[n];
    for (ListDigraph::NodeIt m(TSP); m!= INVALID; ++m){
        ListDigraph::Node y = originalnode[m];
        if (original[n] != original[m]){
            if (x==y){              //Add arcs u,u' with length -M
                deflength[TSP.addArc(n,m)] = -bigM2;
            }
            else{
                if (original[n]==true){     //n is original, m is new node -> add arc from T'
                    ListDigraph::Arc a = al2(y,x);
                    if (a!= INVALID){
                        deflength[TSP.addArc(n,m)] = newlength[a];
                    }
                }
                else{
                    ListDigraph::Arc b = al2(x,y);
                    if (b!= INVALID){
                        deflength[TSP.addArc(n,m)] = newlength[b];
                    }
                }
            }
        }
    }
  }

// Add starting node for open tour
  int extra = 0;
  if ((open == true) && (startingpoint == 0) && (endingpoint == 0)){
    extra = 1;
    ListDigraph::Node xx = TSP.addNode();
    for (ListDigraph::NodeIt n(TSP); n!= INVALID; ++n){
        if (TSP.id(n) != TSP.id(xx)){
            deflength[TSP.addArc(xx,n)] = 0;
            deflength[TSP.addArc(n,xx)] = 0;
        }
    }
  }

//Report results
  std::cout << "TSP is constructed,";
  if (open==false){
    std::cout<< " for a closed tour";
  }
  else{
    std::cout<< " for an open tour";
  }
  std::cout<<std::endl;
  std::cout << "Number of nodes: " << countNodes(TSP) << std::endl;
  std::cout << "Number of arcs: " << countArcs(TSP) << std::endl;

//Check transformation
  if( (countNodes(TSP) - 1*extra == 2* countNodes(ATSP))&&(countArcs(TSP) ==2*countArcs(ATSP)+ 2*countNodes(ATSP) + 2*extra* (countNodes(TSP)- 1))){
    std::cout<<"Transformation ATSP to TSP correct"<< std::endl;
  }
  else{
    std::cout<<"Incorrect transformation: Original # of arcs =" << countArcs(ATSP)<<", original # of nodes="<<countNodes(ATSP) << " New # of arcs = " << countArcs(TSP)<< std::endl;
  }

  digraphWriter(TSP, "TSP.lgf").
    nodeMap("cluster", defcluster).
    arcMap("length", deflength).
    run();
}

void WriteToOutput(void)
{

/* Make distance matrix */
    int sizeD = countNodes(TSP);
    int Distances[sizeD][sizeD];
    ArcLookUp<ListDigraph> al3(TSP);
    for (ListDigraph::NodeIt n(TSP); n!= INVALID; ++n){
        for (ListDigraph::NodeIt m(TSP); m!= INVALID; ++m){
            ListDigraph::Arc a = al3(n,m);
            if (a!= INVALID){
                Distances[TSP.id(n)][TSP.id(m)] = deflength[a]+bigM2;
            }
            else{
                Distances[TSP.id(n)][TSP.id(m)] = 10*bigM2;
            }
        }
    }


// Write TSPLIB file for Concorde
    std::cout<<"Name Concorde file?"<<std::endl;
    std::string name;
    std::getline(std::cin,name);
    std::ofstream tspfile;
    tspfile.open ("tspfile.txt");
    tspfile << "NAME: " << name << "\n";
    tspfile << "TYPE: TSP \n";
    tspfile << "DIMENSION: " << countNodes(TSP) << "\n";
    tspfile << "EDGE_WEIGHT_TYPE: EXPLICIT \n";
    tspfile << "EDGE_WEIGHT_FORMAT: FULL_MATRIX \n";
    tspfile << "EDGE_DATA_FORMAT: EDGE_LIST \n";
    tspfile << "EDGE_DATA_SECTION: \n";
    for (ListDigraph::ArcIt a(TSP); a!= INVALID; ++a){
        tspfile << TSP.id(TSP.source(a)) << " "<< TSP.id(TSP.target(a)) << "\n";
    }
    tspfile << "-1 \n \n";
    tspfile << "EDGE_WEIGHT_SECTION: \n";
    for (int i=0; i<sizeD; i++){
        for (int j=0; j<sizeD; ++j){
            tspfile<< Distances[i][j] << " ";
        }
        tspfile<<"\n";
    }
    tspfile.close();
}

void ReadSolution(bool open, bool TSPvar)
{
//Wait for inputfile
  std::cout<<"Inputfile ready?"<<std::endl;
  std::cin.ignore();

    int sizeInput = countNodes(TSP);
    int sizeS = sizeInput;
    if (open == true){
        sizeS--;
    }
    int newsize = sizeS/2;
    int clustersize = 0;
    int Tsolution[3][sizeInput];      //Tsolution = solution TSP
    std::ifstream inputfile("outputconcorde.txt");
    if(inputfile.is_open()){
        for (int i= 0; i<sizeInput; ++i){
            for (int j = 0; j<3; ++j){
                inputfile >> Tsolution[j][i];
            }
        }
    }
    else{
        std::cout<<"Unable to open file"<<std::endl;
    }

//TSP -> ATSP
    // Delete dummy nodes i+n
    int Asolution[6][newsize];      //Asolution = solution ATSP
    int l=0;
    int ending = sizeS;
    int beginning = sizeS;
    bool ingoing = true;
    for (int i=0; i<sizeInput; ++i){
        if (Tsolution[2][i] != 0){
            Asolution[1][l] = Tsolution[2][i]-bigM2; // Correct for negative distances
            if(Tsolution[1][i]<sizeS/2){
                Asolution[0][l]= Tsolution[1][i];
                if (Tsolution[0][i]==sizeS){
                    ending = Tsolution[1][i];
                }
            }
            else if(Tsolution[0][i]<sizeS/2){
                Asolution[0][l]= Tsolution[0][i];
                ingoing = false;
                if (Tsolution[1][i]==sizeS){
                    beginning = Tsolution[0][i];
                }
            }
            else{
            l--;
            }
            l++;
        }
    }
    for (ListDigraph::NodeIt n(ATSP); n!= INVALID; ++n){
        for(int i= 0; i<newsize; ++i){
            if (G.id(n)==Asolution[0][i]){
                Asolution[2][i] = cluster[n];
                Asolution[3][i] = newline[n];
                Asolution[4][i] = newdirection[n];
                Asolution[5][i] = newtime[n];
            }
        }
    }
    for (int i=0; i<newsize; ++i){
        if ((Asolution[1][i] != 0) || (Asolution[0][i] == beginning) || Asolution[0][i] == ending){
            clustersize++;              //Count clusters
        }
    }

//ATSP -> GTSP
    int SolValue1 = 0;
    int Gsolution[6][clustersize];
    for (int i=0; i<clustersize; i++){
        for (int j=0; j<5; j++){
            Gsolution[j][i] = 0;
        }
    }
  // Delete dummy nodes in cluster
    int k = 0;
    for (int i=0; i<newsize; ++i){
        if ((Asolution[1][i] != 0) || (Asolution[0][i] == beginning) || Asolution[0][i] == ending){
            Gsolution[0][k] = Asolution[0][i];      //node/arc
            if (Asolution[1][i]==0){
                Gsolution[1][k] = Asolution[1][i];
            }
            else{
                Gsolution[1][k] = Asolution[1][i]-bigM; // distance shortest path -> correct for ATSP transformation
            }
            Gsolution[2][k] = Asolution[2][i];  //cluster
            Gsolution[3][k] = Asolution[3][i];  //line
            Gsolution[4][k] = Asolution[4][i];  //direction
            Gsolution[5][k] = Asolution[5][i];  //time
            SolValue1 = SolValue1 + Gsolution[1][k];
            k++;
        }
    }

//Add nodes for solution graph
    digraphReader(SG, "GT.lgf").
    nodeMap("name", solname).
    nodeMap("line", nsolline).
    nodeMap("direction", nsoldirection).
    nodeMap("frequency", solfreq).
    nodeMap("time", soltime).
    //nodeMap("station", station).
    skipArcs().
    run();

if (TSPvar == false){
// Add solution nodes (from GATSP) as arcs to solution graph (=SG)
  for (int i=0; i<clustersize; ++i){            //find for every node the corresponding arc
    for (ListDigraph::ArcIt a(GT); a!= INVALID; ++a){
        if ((group[a]==Gsolution[2][i])&&(arcline[a]==Gsolution[3][i])&&(arcdirection[a]==Gsolution[4][i])&&(arctime[a]==Gsolution[5][i])){ //Time should be correct
            ListDigraph::Arc x = SG.addArc(GT.source(a),GT.target(a));
            sollength[x] = length[a];
            asolline[x] = arcline[a];
            asoldirection[x] = arcdirection[a];
            solgroup[x] = group[a];
            solnontransfer[x] = nontransfer[a];
        }
    }
  }
}

// Add shortest paths (from G) to solution graph (=SG)
  int NumPaths = clustersize;
  if (open == true){
    NumPaths--;
  }
  int PathList[2][NumPaths];

//Construct list with paths
  int a = 0;
  if (ingoing == true){
    for (int i=0; i< clustersize-1; ++i){
        if (Gsolution[0][i+1] != ending){
            PathList[0][a] = Gsolution[0][i];
            PathList[1][a] = Gsolution[0][i+1];
            a++;
        }
    }
    if (Gsolution[0][0] != ending){
        PathList[0][NumPaths-1] = Gsolution[0][clustersize-1];
        PathList[1][NumPaths-1] = Gsolution[0][0];
    }
  }
  else{
    for (int i=0; i< clustersize-1; ++i){
        if (Gsolution[0][i] != beginning){
            PathList[0][a] = Gsolution[0][i+1];
            PathList[1][a] = Gsolution[0][i];
            a++;
        }
    }
    if (Gsolution[0][clustersize-1] != beginning){
        PathList[0][NumPaths-1] = Gsolution[0][0];
        PathList[1][NumPaths-1] = Gsolution[0][clustersize-1];
    }
  }


/* Add Paths */
//Make new Path matrix
  ListDigraph::NodeMap<int> dist(GT);
  Dijkstra<ListDigraph> dijkstra(GT, length);
  dijkstra.distMap(dist);
  Path<ListDigraph> NewPathMatrix[NumPaths];
if (TSPvar == false){
  ListDigraph::Node source;
  ListDigraph::Node target;
  for (int c=0; c<NumPaths; c++){
    for (ListDigraph::NodeIt i(GATSP); i!= INVALID; ++i){
        for (ListDigraph::NodeIt j(GATSP); j!= INVALID; ++j){
            if ((PathList[0][c] == GATSP.id(i)) && (PathList[1][c] == GATSP.id(j))){
                source = i;
                target = j;
                ListDigraph::Node n = nodetarget[i];
                ListDigraph::Node m = nodesource[j];
                dijkstra.run(n,m);
                NewPathMatrix[c] = dijkstra.path(m);
            }
        }
    }
  }
}
else{
  for (int c=0; c<NumPaths; c++){
    for (ListDigraph::NodeIt n(GT); n!= INVALID; ++n){
        for (ListDigraph::NodeIt m(GT); m!= INVALID; ++m){
            if ((PathList[0][c] == GT.id(n)) && (PathList[1][c] == GT.id(m))){
                dijkstra.run(n,m);
                NewPathMatrix[c] = dijkstra.path(m);
            }
        }
    }
  }
}
// Add them to graph
  for (int i=0; i<NumPaths; ++i){
    for (Path<ListDigraph>::ArcIt a(NewPathMatrix[i]); a!= INVALID; ++a){
        ListDigraph::Arc x = SG.addArc(GT.source(a),GT.target(a));
        sollength[x] = length[a];
        asolline[x] = arcline[a];
        asoldirection[x] = arcdirection[a];
        solgroup[x] = group[a];
        solnontransfer[x] = nontransfer[a];
    }
  }


  int SolValue2 = 0;
  for (ListDigraph::ArcIt a(SG); a!= INVALID; ++a){
    SolValue2 = SolValue2 + sollength[a];
  }

/* Report results */
  std::cout << "The solution graph is constructed" << std::endl;
  std::cout << "Number of nodes: " << countNodes(SG) << std::endl;
  std::cout << "Number of arcs: " << countArcs(SG) << std::endl;

  // Check solution values
  if (open == false){
    if (SolValue1 == SolValue2){
        std::cout<<"Solution correct: "<<SolValue2<<std::endl;
    }
    else{
        std::cout<<"Solution incorrect: Should be "<<SolValue1<<", but is "<<SolValue2<<std::endl;
    }
  }
  else{
    std::cout<<"Solution value: "<<SolValue2<<std::endl;
  }

}

void VerificationCheck(bool open)
{
// Check indegree and outdegree
    int outgoing = 0;
    int ingoing = 0;
    int uneven = 0;
    int unused = 0;
    ListDigraph::Node beginnode;
    ListDigraph::Node endnode;
    for (ListDigraph::NodeIt n(SG); n!= INVALID; ++n){
        outgoing = 0;
        ingoing = 0;
        for (ListDigraph::OutArcIt a(SG, n); a!= INVALID; ++a){
            outgoing++;
        }
        for (ListDigraph::InArcIt b(SG, n); b!= INVALID; ++b){
            ingoing++;
        }
        if (ingoing != outgoing){
            uneven++;
            if (ingoing<outgoing){
                beginnode = n;
            }
            else{
                endnode = n;
            }
        }
        if (ingoing + outgoing == 0){
            unused++;
        }
    }

    if (open ==true){
        if (uneven ==2){
            std::cout<<"In- and outdegree correct"<<std::endl;
        }
        else{
            std::cout<<"In- and outdegree incorrect: "<<uneven<<" uneven nodes"<<std::endl;
        }
    }
    else{
        if (uneven ==0 ){
            std::cout<<"In- and outdegree correct"<<std::endl;
        }
        else{
            std::cout<<"In- and outdegree incorrect: "<<uneven<<" uneven nodes"<<std::endl;
        }
    }

// Make ordering
    int solArcs = countArcs(SG);
    ListDigraph::Node findNode = beginnode;
    ListDigraph::Node findNode2 = endnode;
    int i=1;
    int j=solArcs;
    bool last = true;           // whether the last arc was a non transfer arc
    bool last2 = true;
    bool change = true;
    while ((findNode != findNode2) && (change == true) ){
        change = false;
        for (ListDigraph::ArcIt a(SG); a!= INVALID; ++a){
            if ((SG.source(a) == findNode) && (solorder[a]==0) && (solnontransfer[a] || last)){
                solorder[a] = i;
                findNode = SG.target(a);
                i++;
                if(solnontransfer[a] == false){
                    last = false;
                }
                else{
                    last = true;
                }
                change = true;
            }
            if ((SG.target(a) == findNode2)&&(solorder[a]==0) && (solnontransfer[a] || last2)){
                solorder[a] = j;
                findNode2 = SG.source(a);
                j--;
                if(solnontransfer[a] == false){
                    last2 = false;
                }
                else{
                    last2 = true;
                }
                change = true;
            }
        }
    }
// If no tour has been found also use consecutive transfer arcs
    if (findNode != findNode2){
        change = true;
        while ((findNode != findNode2) && (change == true) ){
            change = false;
            for (ListDigraph::ArcIt a(SG); a!= INVALID; ++a){
                if ((SG.source(a) == findNode) && (solorder[a]==0)){
                    solorder[a] = i;
                    findNode = SG.target(a);
                    i++;
                    change = true;
                }
                if ((SG.target(a) == findNode2)&&(solorder[a]==0)){
                    solorder[a] = j;
                    findNode2 = SG.source(a);
                    j--;
                    change = true;
                }
            }
        }
    }

// Make second ordering of arcs left
    ListDigraph::Arc newArc;
    int arcsLeft=0;
    if (j-i > 1){
        i=1;
        for (ListDigraph::ArcIt a(SG); a!=INVALID; ++a){
            if (solorder[a] == 0){
                newArc = a;
                arcsLeft++;
            }
        }
        last = solnontransfer[newArc];
        solorder[newArc] = countArcs(SG) + 100;
        findNode = SG.target(newArc);
        for (int cnt=0; cnt<arcsLeft; cnt++){
            for (ListDigraph::ArcIt a(SG); a!= INVALID; ++a){
                if ((SG.source(a) == findNode) && (solorder[a] == 0) && (solnontransfer[a] || last)){
                    solorder[a] = countArcs(SG) + 100 + i ;
                    findNode = SG.target(a);
                    i++;
                    if(solnontransfer[a] == false){
                        last = false;
                    }
                    else{
                        last = true;
                    }
                }
            }
        }
    }

  //Find sources and targets of all arcs
  for (ListDigraph::ArcIt a(SG); a!=INVALID; ++a){
    solsource[a] = solname[SG.source(a)];
    soltarget[a] = solname[SG.target(a)];
    solsourcetime[a] = soltime[SG.source(a)];
    soltargettime[a] = soltime[SG.target(a)];
  }

  // Write solution value
  digraphWriter(SG, "Solution.lgf").
    skipNodes().
    arcMap("Group", solgroup).
    arcMap("Direction", asoldirection).
    arcMap("Order", solorder).
    arcMap("Line", asolline).
    arcMap("Length", sollength).
    arcMap("Departure", solsource).
    arcMap("Arrival", soltarget).
    arcMap("DepTime", solsourcetime).
    arcMap("ArrTime", soltargettime).
    run();


    for (ListDigraph::NodeIt n(SG); n!=INVALID; ++n){
        for (ListDigraph::NodeIt m(SG); m!=INVALID; ++m){
            if ((endnode == n) &&(beginnode == m)){
                SG.addArc(n,m);
            }
        }
    }

// Check strongly connected component
int components = countStronglyConnectedComponents(SG);
    if (components - unused == 1){
        std::cout<<"Solution graph connected"<<std::endl;
    }
    else{
        std::cout<<"Solution graph not connected: "<<countStronglyConnectedComponents(SG)<<" components, "<<unused<<" unused nodes"<<std::endl;
    }

}

int main(int argc, char* argv[])
{
  bool op = true;
  bool TSP_Variant = false; 
  int startp = 0;      // Let's you choose starting point if it's 0 then no point is chosen
  int endp = 0;       // Let's you choose ending point if it's 0 then no point is chosen
  
  int timeInt = 20;      // Choose the time interval for the timetable representation. If timeInterval == 1, then the non extended version is used
  int transferT = 5;    //Transfer times representation
   // Does only have to be specified when non extended version is used
   //Transfer times exceptions
  int lbT = 0;// Lower bound of all transfer times
  
  std::vector<int[2]> exceptionStations; // =   {{ , }, ... , { , }};           // Input format: {station, lower bound transfer time}
  int uTransfer = 0;   // States transfer times that are unwanted, for example 1
  int weightUnwantedTransfer = 0; // Gives a weight to the unwanted transfer time stated above to avoid using it too often

  
  std::string inputstart = "Network_arcs.lgf";          // Input file with all arcs;  
  std::string inputGDRPP = "Network_nodes.lgf";    	// Input file with all nodes
  
  for(int i = 1; i < argc; ++i)
  {
    char *token;
    token = std::strtok(argv[i], "-");
    std::vector<std::string> tokenVector;
    while (token != NULL) {
        tokenVector.push_back(token);
        token = std::strtok(NULL, "-");
    }
    if (tokenVector.size()==1)
    {
      std::string option = tokenVector.at(0);
      if(option == "closed")
	op = false;
      if(option == "tsp")
	TSP_Variant = true;
    }
    if(tokenVector.size()==2)
    {
      std::string str = tokenVector.at(0);
      if(str=="a")
	inputstart = tokenVector.at(1);
      if(str=="n")
	inputGDRPP = tokenVector.at(1);
      if(str=="tI")
	timeInt = atoi(tokenVector.at(1).c_str());
      if(str=="cT")
	transferT = atoi(tokenVector.at(1).c_str());
      if(str == "start")
	startp = atoi(tokenVector.at(1).c_str());
      if(str=="end")
	endp = atoi(tokenVector.at(1).c_str());
      if(str=="lb")
	lbT = atoi(tokenVector.at(1).c_str());
    }
    
  }
  std::cout<<"The chosen options are: "<<std::endl;
  if(TSP_Variant)
    std::cout<<"Visit all stations (TSP-Variant)";
  else
    std::cout<<"Visit all connections";
  if(op)
    std::cout<<"in an open tour."<<std::endl;
  else
    std::cout<<" in a closed tour."<<std::endl;
  if(timeInt ==1)
    std::cout <<"Variant with a constant transfer time of"<<transferT<<" minutes"<<std::endl;
  else
  {
    std::cout <<"The time interval is "<<timeInt<<" minutes and the lower bound for the transfers is "<<lbT<<std::endl;
  }
  
// Transformation to Starting graph
    GSTART(inputstart);
// Transformation to GDRPP
    GSTARTtoGDRPP(inputGDRPP, TSP_Variant);
// Transformation to GDRPP extended
    GDRPPtoGDRPPextended(timeInt, transferT, lbT, exceptionStations, uTransfer, weightUnwantedTransfer);
// Transformation to GATSP
    GDRPPtoGATSP(TSP_Variant);
// Transformation to ATSP
    GATSPtoATSP(op, TSP_Variant, startp, endp);
// Transformation to TSP
    ATSPtoTSP(op, startp, endp);
// Write to Output
    WriteToOutput();
// Read Concorde solution file
    ReadSolution(op, TSP_Variant);
// Check solution graph
    VerificationCheck(op);

    return 0;
}
